package inf101.v18.connectfour.board;

import inf101.v18.connectfour.events.winning.IWinCondition;
import inf101.v18.connectfour.objects.IGameObject;
import inf101.v18.connectfour.objects.IPlayerObject;
import inf101.v18.connectfour.player.IPlayer;

import java.util.ArrayList;
import java.util.List;

/**
 * A board takes care of players, interacting with the grid and win-conditions.
 */
public class Board implements IBoard{
    IGrid<IGameObject> grid;
    List<IPlayer> players;
    int playersTurn = 0;
    final List<IWinCondition> conditionList;
    boolean won = false;
    final IGameObject empty;

    public Board(int width, int height, IGameObject empty, List<IWinCondition> conditions) {
        grid = new Grid<>(width, height, empty);
        players = new ArrayList<>();
        conditionList = conditions;
        this.empty = empty;
    }

    /**
     * Makes sure we don't have two equal players.
     *
     * @param player    The new player to test
     * @return          True if the player is non-unique
     */
    private boolean playerExists(IPlayer player) {
        for (IPlayer p : players) {
            if (player.equals(p)) {
                return true;
            }
        }
        return false;
    }

    /**
     * Checks if anyone has won.
     *
     * @param x The x-coordinate of the last dropped object
     * @param y The y-coordinate of the last dropped object
     */
    private void checkWin(int x, int y) {
        for (IWinCondition con : conditionList) {
            if (con.hasWon(x, y, this)) {
                won = true;
            }
        }
    }

    @Override
    public boolean addPlayer(IPlayer p) {
        if (!playerExists(p)) {
            players.add(p);
            return true;
        } else {
            return false;
        }
    }

    @Override
    public ArrayList<IPlayer> getPlayers() {
        return new ArrayList<>(players);
    }

    @Override
    public IPlayer nextPlayer() {
        if (players.isEmpty()) {
            return null;
        }
        IPlayer p = players.get(playersTurn);
        this.playersTurn = this.playersTurn + 2 > this.players.size() ? 0 : this.playersTurn + 1;
        return p;
    }

    @Override
    public boolean isFull() {
        return grid.isFull();
    }

    @Override
    public boolean won() {
        return won;
    }

    @Override
    public void flush() {
        grid.flush();
        playersTurn = 0;
        won = false;
    }

    @Override
    public int getHeight() {
        return grid.getHeight();
    }

    @Override
    public int getWidth() {
        return grid.getWidth();
    }

    @Override
    public void set(int x, int y, IGameObject element) {
        grid.set(x, y, element);
        if (element instanceof IPlayerObject) {
            checkWin(x, y);
        }
    }

    @Override
    public IGameObject get(int x, int y) { return grid.get(x, y); }

    @Override
    public IGrid<IGameObject> copy() {
        return grid.copy();
    }

    @Override
    public IBoard copyAll() {
        Board newBoard = new Board(getWidth(), getHeight(), empty, conditionList);
        newBoard.players = new ArrayList<>(players);
        newBoard.won = won;
        newBoard.grid = copy();
        newBoard.playersTurn = playersTurn;
        return newBoard;
    }
}