package inf101.v18.connectfour.events.random;

import inf101.v18.connectfour.board.IVerticalBoard;

import java.util.Random;

/**
 * Contains basic probability methods usable by all random events.
 */
public abstract class RandomEvent implements IRandomEvent {
    final Random random = new Random();

    @Override
    public boolean flipCoin(IVerticalBoard b) {
        if (random.nextInt(2) > 0) {
            act(b);
            return true;
        }
        return false;
    }

    @Override
    public boolean rollDie(IVerticalBoard b, int n) {
        if (random.nextInt((int)Math.pow(6, n)) < 1) {
            act(b);
            return true;
        }
        return false;
    }

    /**
     * Does something depending on the event.
     *
     * @param b The board to act on
     */
    abstract void act(IVerticalBoard b);
}
