package inf101.v18.connectfour.events.winning;

import inf101.v18.connectfour.board.IBoard;

import java.awt.*;

/**
 * Contains methods for implementing any n in a row rules.
 */
abstract class InARow implements IWinCondition {
    /**
     * Checks if the supplied position has enough neighbours in a straight line of the same type
     *
     * @param x     The x-coordinate of the start position
     * @param y     The y-coordinate of the start position
     * @param board The board on which to perform the task
     * @param t     The number of elements in a row, which constitutes a win (excluding start position)
     * @return      True if x, y was a winning move
     */
    static boolean hasWon(int x, int y, IBoard board, int t) {
        return count(x, y, 1, 0, board) + count(x, y, -1, 0, board) >= t ||
                count(x, y, 0, -1, board) + count(x, y, 0, 1, board) >= t ||
                count(x, y, 1, 1, board) + count(x, y, -1, -1, board) >= t ||
                count(x, y, 1, -1, board) + count(x, y, -1, 1, board) >= t;
    }

    /**
     * Counts all elements from a start position in a set direction with a matching color.
     *
     * @param x     The x-coordinate of the start position
     * @param y     The y-coordinate of the start position
     * @param iX    Step amount in x-direction
     * @param iY    Step amount in y-direction
     * @param b     The board on which to perform the task
     * @return      The number of elements found
     */
    private static int count(int x, int y, int iX, int iY, IBoard b) {
        int count = 0;
        Color target = b.get(x, y).getColor();
        while ((y + iY >= 0 && y + iY < b.getHeight()) && (x + iX >= 0 && x + iX < b.getWidth()) && b.get(x + iX, y + iY).getColor() == target) {
            x += iX;
            y += iY;
            count++;
        }
        return count;
    }
}
