package inf101.v18.connectfour.interfaces;

import inf101.v18.connectfour.board.IVerticalBoard;
import inf101.v18.connectfour.board.VerticalBoard;
import inf101.v18.connectfour.events.winning.FourInARow;
import inf101.v18.connectfour.events.winning.IWinCondition;
import inf101.v18.connectfour.objects.Empty;
import inf101.v18.connectfour.objects.IGameObject;
import inf101.v18.connectfour.objects.Token;
import inf101.v18.connectfour.player.AI;
import inf101.v18.connectfour.player.Player;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

/**
 * A CLI implementation of connect four.
 */
public class ConnectFourCLI extends CLI implements IVerticalGameInterface {
    public ConnectFourCLI(int width, int height) {
        List<IWinCondition> con = new ArrayList<>();
        con.add(new FourInARow());
        board = new VerticalBoard(width, height, new Empty(), con);
        board.addPlayer(new Player(readName(1), Color.green));
        if (playingAlone()) {
            board.addPlayer(new AI("AI", Color.red));
        } else {
            board.addPlayer(new Player(readName(2), Color.red));
        }
        currentPlayer = board.nextPlayer();
    }

    @Override
    public void displayBoard() {
        for (int i = 1; i <= board.getWidth(); i++) {
            System.out.print(i + " ");
        }
        System.out.println();
        for (int i = board.getHeight() - 1; i > -1; i--) {
            for (int j = 0; j < board.getWidth(); j++) {
                IGameObject obj = board.get(j,i);
                System.out.print(colorToAnsi(obj.getColor(), 'O') + " ");
            }
            if (i > 0) {
                System.out.println();
            }
        }
    }

    @Override
    public boolean placePlayerObject(int x, int y) {
        return false;
    }

    @Override
    public boolean dropObject(int i) {
        boolean hasDropped = ((IVerticalBoard)board).drop(i, new Token(currentPlayer));
        if (!hasDropped) {
            return false;
        }
        if (board.won()) {
            displayBoard();
            displayMessage("\n" + currentPlayer.getName() + " won");
        }
        currentPlayer = board.nextPlayer();
        return true;
    }
}
