package inf101.v18.connectfour.interfaces;

import inf101.v18.connectfour.board.Board;
import inf101.v18.connectfour.events.winning.IWinCondition;
import inf101.v18.connectfour.events.winning.ThreeInARow;
import inf101.v18.connectfour.objects.Empty;
import inf101.v18.connectfour.objects.IGameObject;
import inf101.v18.connectfour.objects.IPlayerObject;
import inf101.v18.connectfour.objects.Token;
import inf101.v18.connectfour.player.AI;
import inf101.v18.connectfour.player.Player;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

/**
 * A Tic-Tac-Toe implementation using a Command Line Interface.
 */
public class TicTacToeCLI extends CLI {
    public TicTacToeCLI() {
        List<IWinCondition> con = new ArrayList<>();
        con.add(new ThreeInARow());
        board = new Board(3, 3, new Empty(), con);
        board.addPlayer(new Player(readName(1), Color.green));
        if (playingAlone()) {
            board.addPlayer(new AI("AI", Color.red));
        } else {
            board.addPlayer(new Player(readName(2), Color.red));
        }
        currentPlayer = board.nextPlayer();
    }

    @Override
    public void displayBoard() {
        for (int i = board.getHeight() - 1; i >= 0; i--) {
            System.out.print((i + 1) + " ");
            for (int j = 0; j < board.getWidth(); j++) {
                IGameObject obj = board.get(j,i);
                System.out.print(colorToAnsi(obj.getColor(), getSymbol(obj.getColor())) + " ");
            }
            System.out.println();
        }
        System.out.print(" ");
        for (int i = 0; i < board.getWidth(); i++) {
            System.out.print(" " + (i + 1));
        }
    }

    @Override
    public boolean placePlayerObject(int x, int y) {
        if (board.get(x, y) instanceof IPlayerObject) {
            return false;
        }
        try {
            board.set(x, y, new Token(currentPlayer));
            if (board.won()) {
                displayBoard();
                displayMessage("\n" + currentPlayer.getName() + " won");
            }
            currentPlayer = board.nextPlayer();
            return true;
        } catch (IllegalArgumentException e) {
            return false;
        }
    }
}
