package inf101.v18.connectfour;

import inf101.v18.connectfour.board.Board;
import inf101.v18.connectfour.board.IBoard;
import inf101.v18.connectfour.board.IVerticalBoard;
import inf101.v18.connectfour.board.VerticalBoard;
import inf101.v18.connectfour.events.winning.FourInARow;
import inf101.v18.connectfour.events.winning.IWinCondition;
import inf101.v18.connectfour.objects.*;
import inf101.v18.connectfour.player.AI;
import inf101.v18.connectfour.player.IAI;
import inf101.v18.connectfour.player.IPlayer;
import inf101.v18.connectfour.player.Player;
import org.junit.Assert;
import org.junit.Test;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

import static junit.framework.TestCase.fail;
import static org.junit.Assert.assertEquals;

public class AITest {
    @Test
    public void dropTest() {
        try {
            IVerticalBoard board = new VerticalBoard(5, 5, new Empty(), new ArrayList<>());
            IPlayerObject token = new Token(new Player("Playa", Color.MAGENTA));
            IAI ai = new AI("AI", Color.yellow);
            board.addPlayer(ai);
            for (int i = 0; i < 5; i++) {
                board.drop(1, new Immovable());
            }
            for (int i = 0; i < 20; i++) {
                int choice = ai.getColumn(board);
                Assert.assertNotEquals(1, choice);
                board.drop(choice, token);
            }
        } catch (IllegalArgumentException e) {
            fail();
        }
    }

    @Test
    public void placeTest() {
        try {
            IBoard board = new Board(5, 5, new Empty(), new ArrayList<>());
            IPlayerObject token = new Token(new Player("Playa", Color.MAGENTA));
            IAI ai = new AI("AI", Color.yellow);
            board.addPlayer(ai);
            int[] choice = ai.getXY(board);
            assertEquals(2, choice.length);
            for (int i = 0; i < 5 * 5; i++) {
                choice = ai.getXY(board);
                board.set(choice[0], choice[1], token);
            }
        } catch (IllegalArgumentException e) {
            fail();
        }
    }

    @Test
    public void winTest() {
        List<IWinCondition> con = new ArrayList<>();
        con.add(new FourInARow());
        IVerticalBoard board = new VerticalBoard(5, 5, new Empty(), con);
        IAI ai = new AI("AI", Color.red);
        board.addPlayer(ai);
        for (int i = 0; i < 3; i++) {
            board.set(i, 0, new Token(ai));
        }
        assertEquals(3, ai.getColumn(board));
    }

    @Test
    public void blockTest() {
        List<IWinCondition> con = new ArrayList<>();
        con.add(new FourInARow());
        IVerticalBoard board = new VerticalBoard(5, 5, new Empty(), con);
        IPlayer player = new Player("AAA", Color.green);
        IAI ai = new AI("AI", Color.red);
        board.addPlayer(player);
        board.addPlayer(ai);
        for (int i = 0; i < 3; i++) {
            board.set(i, 0, new Token(player));
        }
        assertEquals(3, ai.getColumn(board));
    }
}
