package inf101.v18.connectfour;

import inf101.v18.connectfour.board.*;
import inf101.v18.connectfour.events.winning.FourInARow;
import inf101.v18.connectfour.events.winning.IWinCondition;
import inf101.v18.connectfour.objects.*;
import inf101.v18.connectfour.player.IPlayer;
import inf101.v18.connectfour.player.Player;
import org.junit.Test;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.*;

public class BoardTest {
    @Test
    public void setGetTest() {
        IBoard board = new Board(5, 5, new Empty(), new ArrayList<>());
        IPlayerObject token = new Token(new Player("Playa", Color.MAGENTA));
        board.set(3, 3, token);
        assertNotEquals(token, board.get(2, 3));
        assertEquals(token, board.get(3, 3));
    }

    @Test(expected = IllegalArgumentException.class)
    public void heightTooBigTest() {
        IBoard board = new Board(5, 5, new Empty(), new ArrayList<>());
        IPlayerObject token = new Token(new Player("Playa", Color.MAGENTA));
        board.set(0, 5, token);
    }

    @Test(expected = IllegalArgumentException.class)
    public void widthTooSmallTest() {
        IBoard board = new Board(5, 5, new Empty(), new ArrayList<>());
        IPlayerObject token = new Token(new Player("Playa", Color.MAGENTA));
        board.set(-1, 1, token);
    }

    @Test(expected = IllegalArgumentException.class)
    public void heightTooSmallTest() {
        IBoard board = new Board(5, 5, new Empty(), new ArrayList<>());
        IPlayerObject token = new Token(new Player("Playa", Color.MAGENTA));
        board.set(0, -2, token);
    }

    @Test(expected = IllegalArgumentException.class)
    public void widthTooBigTest() {
        IBoard board = new Board(5, 5, new Empty(), new ArrayList<>());
        IPlayerObject token = new Token(new Player("Playa", Color.MAGENTA));
        board.set(5, 3, token);
    }

    @Test
    public void heightWidthTest() {
        IBoard board = new Board(5, 6, new Empty(), new ArrayList<>());
        assertEquals(5, board.getWidth());
        assertEquals(6, board.getHeight());
    }

    @Test
    public void addPlayerTest() {
        IBoard board = new Board(5, 6, new Empty(), new ArrayList<>());
        IPlayer player = new Player("ppp", Color.yellow);
        assertEquals(0, board.getPlayers().size());
        assertTrue(board.addPlayer(player));
        assertEquals(player, board.getPlayers().get(0));
        assertFalse(board.addPlayer(new Player("ppp", Color.MAGENTA)));
        assertFalse(board.addPlayer(new Player("asdaf", Color.yellow)));
    }

    @Test
    public void nextPlayerTest() {
        IBoard board = new Board(1, 1, new Empty(), new ArrayList<>());
        IPlayer player1 = new Player("Nem", Color.yellow);
        IPlayer player2 = new Player("Men", Color.ORANGE);
        board.addPlayer(player1);
        board.addPlayer(player2);
        assertEquals(player1, board.nextPlayer());
        assertEquals(player2, board.nextPlayer());
        assertEquals(player1, board.nextPlayer());
    }

    @Test
    public void onePlayerTest() {
        IBoard board = new Board(1, 1, new Empty(), new ArrayList<>());
        IPlayer player1 = new Player("Nem", Color.yellow);
        board.addPlayer(player1);
        assertNotNull(board.nextPlayer());
        assertEquals(player1, board.nextPlayer());
        assertEquals(player1, board.nextPlayer());
    }

    @Test
    public void noPlayersTest() {
        IBoard board = new Board(1, 1, new Empty(), new ArrayList<>());
        assertNull(board.nextPlayer());
    }

    @Test
    public void flushTest() {
        IGameObject empty = new Empty();
        List<IWinCondition> con = new ArrayList<>();
        con.add(new FourInARow());
        IBoard board = new Board(4, 5, empty, con);
        IPlayer player1 = new Player("ÆÆÆ", Color.RED);
        IPlayer player2 = new Player("asdÆ", Color.GREEN);
        board.addPlayer(player1);
        board.addPlayer(player2);
        IPlayerObject token = new Token(player1);
        board.set(0, 0, token);
        board.set(1, 1, token);
        board.set(2, 2, token);
        board.set(3, 3, token);
        assertTrue(board.won());
        board.nextPlayer();
        assertEquals(player2, board.nextPlayer());
        board.flush();
        assertFalse(board.won());
        assertEquals(player1, board.nextPlayer());
        assertEquals(empty, board.get(0, 0));
        assertEquals(empty, board.get(1, 1));
        assertEquals(empty, board.get(2, 2));
        assertEquals(empty, board.get(3, 3));
    }

    @Test
    public void isFullTest() {
        IBoard board = new Board(5, 5, new Empty(), new ArrayList<>());
        IGameObject token = new Token(new Player("Naem", Color.green));
        assertFalse(board.isFull());
        for (int i = 0; i < 5; i++) {
            board.set(0, i, token);
        }
        assertFalse(board.isFull());
        for (int i = 0; i < 5; i++) {
            board.set(1, i, token);
        }
        assertFalse(board.isFull());
        for (int i = 0; i < 5; i++) {
            board.set(2, i, token);
        }
        assertFalse(board.isFull());
        for (int i = 0; i < 5; i++) {
            board.set(3, i, token);
        }
        assertFalse(board.isFull());
        for (int i = 0; i < 5; i++) {
            board.set(4, i, token);
        }
        assertTrue(board.isFull());
    }
}
